'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var entity = require('../../dist/entity-99e10029.cjs.js');
var storage_js = require('class-transformer/cjs/storage.js');

/* eslint-disable @typescript-eslint/no-var-requires */
const camelCase = require("lodash/camelCase");
const snakeCase = require("lodash/snakeCase");
function getExposeMetadataOptions(propertyName, options = {}) {
  const exposeOptions = {};
  switch (options.nameCasing) {
    case "camelCase":
      exposeOptions.name = camelCase(propertyName);
      break;
    case "snakeCase":
      exposeOptions.name = snakeCase(propertyName);
      break;
  }
  return exposeOptions;
}
function ExposeAll(options) {
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  return function (target) {
    const object = new target();
    Object.keys(object).forEach(propertyName => {
      const exposeMetadata = storage_js.defaultMetadataStorage.findExposeMetadata(target, propertyName);
      storage_js.defaultMetadataStorage.addExposeMetadata({
        target,
        propertyName,
        options: {
          ...getExposeMetadataOptions(propertyName, options),
          ...exposeMetadata?.options
        }
      });
    });
  };
}

function Union({
  discriminatorProperty = "type",
  subTypes = []
}) {
  return function (target, propertyKey) {
    const reflectedType = Reflect.getMetadata("design:type", target, propertyKey);
    storage_js.defaultMetadataStorage.addTypeMetadata({
      target: target.constructor,
      propertyName: propertyKey,
      reflectedType,
      typeFunction: () => entity.Union,
      options: {
        discriminator: {
          property: discriminatorProperty,
          subTypes
        }
      }
    });
    if (reflectedType.name === "Array") {
      storage_js.defaultMetadataStorage.addTransformMetadata({
        target: target.constructor,
        propertyName: propertyKey,
        transformFn: ({
          value
        }) => value?.filter(v => !(v instanceof entity.Union)),
        options: {}
      });
    }
  };
}

exports.Entity = entity.Entity;
exports.HasEntity = entity.HasEntity;
exports.ExposeAll = ExposeAll;
exports.Union = Union;
